#include "gestioneStadio.h"
#include <stdio.h>       /* per usare  input-output */
#include <stdlib.h>      /* per EXIT_FAILURE */
#include <sys/socket.h>  /* per usare socket */
#include <sys/types.h>
#include <netinet/in.h>
#include <string.h>
#include <netdb.h>
#include <errno.h>        /* gestione degli errori */

//stampa i comandi disponibili per l'utente
void stampaComandi(){
	 printf("\n");
	 printf("********************** <CLIENT %d> **********************\n",getpid());
	 printf("Menu' dei comandi:\n");
     printf("-- Seleziona una settore (1-%d) e il numero di posti adiacenti che vuoi prenotare\n",N_SETTORI);
     printf("-- Scrivi 0 o un carattere alfanumerico per uscire\n");
     printf("\n");
}

//legge dal server le informazioni con la descrizione
void readInfoFromserver(int sock){
	 int i; //contatore
	 char descrizione[N_SETTORI][256]; 	
  
     //lettura	
     if( read(sock,(char *)&descrizione, sizeof(descrizione)) < 0 ) {
     	printf("<CLIENT %d - ERR> Errore %s durante la read dalla socket\n",getpid(),strerror(errno));
			exit(1);
     }
     for (i = 0; i < N_SETTORI; i++) {
      	printf("°°° %s", descrizione[i]);
     }
     printf("\n");
}

//acquisisce l'input sulle prenotazioni
//da standard input nel caso di client normale (mode=1)
//predefinito nel caso di test concreto (mode=2)
void prendiPrenotazione(int mode, int prenotazione[2]){
	//parametro locale
	char lettura[2];
	
	switch(mode){
			//caso 1: normale
			case 0:
				//prendo l'input dell'utente
				printf("<CLIENT %d - INPUT> Inserire il codice del settore:\n",getpid());
				scanf("%s",&lettura);
				prenotazione[0] = atoi(lettura);//se inserisce lettere legge 0
		
				//condizione d'uscita
				if (prenotazione[0]==0) 
					break;
		
				//leggo il numero di posti adiacenti
				printf("<CLIENT %d - INPUT> Inserire il numero di posti adiacenti desiderati:\n",getpid());
				scanf("%s",&lettura);
				prenotazione[1] = atoi(lettura);
				break;
			//test concreto
			case 1:
				//prenotazione di una fila in tribuna d'onore
				//la seconda volta esce
				if(prenotazione[0]==6)
					prenotazione[0]=0;
				else{
					prenotazione[0]=6;
					prenotazione[1]=20;
				}
				break;
			//test randomico
			case 2:
				//prenotazione di una fila random
				//la seconda volta esce
				if(prenotazione[0]>0 && prenotazione[0]<10)
					prenotazione[0]=0;
				else{
					srandom(getpid());
					prenotazione[0]=(random() % 9)+1;
					prenotazione[1]=20;
				}
				break;
	}
}

//processo client
main(int argc, char** argv) {
	
	//parametri locali
	int prenotazione[2];
	int mode;
	//parametri per usare le socket
	int sock;
	struct sockaddr_in server;
	struct hostent *hp;
	//per colloquiare col server
	char result[256];
	
	//controllo i parametri
	if(argc!=4) {
		printf("Devi specificare: <host> <numero-della-porta> <tipo-client (0,1 o 2)\n");
		exit(1);
	}
	
	//leggo il tipo del client
	mode = atoi(argv[3]);
	
	// chiedo al resolver di tradurre il nome in indirizzo ip 
	//se argv[1] e’ gia’ un indirizzo, allora hp viene semplicemente aggiornato con l’indirizzo fornito
	hp = gethostbyname(argv[1]);
	if( hp == NULL ) {
		printf("<ERR> Host %s sconosciuto.\n", argv[1]);
		exit(1);
	}
	//richiedo la socket al sistema operativo
	sock = socket(AF_INET, SOCK_STREAM, 0);
	
	if( sock < 0 ) {
		printf("<CLIENT %d - ERR> Errore %s nella creazione della socket\n",getpid(),strerror(errno));
		exit(1);
	}
	server.sin_family = AF_INET;
	bcopy(hp->h_addr, &server.sin_addr, hp->h_length);
	server.sin_port = htons(atoi(argv[2]));
	
	// connessione
	if( connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0 ) {
		printf("<CLIENT %d - ERR> Errore %s durante la connect\n",getpid(),strerror(errno));
		exit(1);
	}
	printf("<CLIENT %d - INFO> Numero socket %d\n",getpid(),sock);
	printf("<CLIENT %d - INFO> Connesso a %s, porta %d\n",getpid(),argv[1],ntohs(server.sin_port));
	
	//presentazione
	stampaComandi();
	readInfoFromserver(sock);
	
	while(1){
		
		//acquisisco l'input
		prendiPrenotazione(mode, prenotazione);
		
		//condizione d'uscita
		if (prenotazione[0]==0) 
			break;
			
		//scelta del settore e del numero di posti adiacenti
		if( write(sock,&prenotazione, sizeof(prenotazione) ) <0 ) {
			printf("<CLIENT %d - ERR> Errore %s durante la write nella socket\n",getpid(),strerror(errno));
			exit(1);
		}
		else {
			//leggo il risultato dal server
			if( read(sock,&result, sizeof(result)) < 0 )	 {
				printf("<CLIENT %d - ERR> Errore %s durante la read dalla socket\n",getpid(),strerror(errno));
				exit(1);
			}
			printf("\n----> <CLIENT %d - INFO>\n%s\n",getpid(),result);
		}
		//leggo la nuova situazione dei settori
		readInfoFromserver(sock);
	}
	close(sock);
	printf("<CLIENT %d - INFO> Grazie e Buona Visione!!!\n",getpid());
	exit(0);
}




