#include "stadioOlimpico.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>

//nomi dei settori
char* nomiSettori[N_SETTORI] = {"Curva Sud", "Distinti Sud", "Distinti Ovest", "Tribuna Tevere", 
	"Tribuna Monte Mario", "Tribuna d'Onore", "Curva Nord", "Distinti Est", "Distinti Nord"};

/*
 * Metodo di supporto all'inizializzazione dello stadio: alloca lo spazio nello heap
 * per le file del settore i-esimo e per i posti di tali file.
 * Inizializza file e posti del settore i-esimo. 
 */
void creaFilePosti(int i, int nfile, int npostifila, int costoIntero){
	int j,k;
	settori[i].costoIntero = costoIntero;
	settori[i].nFile = nfile;
	settori[i].file = malloc(sizeof(struct fila[nfile]));
	for (j = 0; j < nfile; j++){
		settori[i].file[j].nPosti = npostifila;
		settori[i].file[j].posti = malloc(sizeof(struct posto[npostifila]));
		for (k = 0; k < npostifila; k++){
			settori[i].file[j].posti[k].codice = k;
			settori[i].file[j].posti[k].is_prenotato = 0;
		}
		settori[i].file[j].codice = j;
		settori[i].file[j].nPostiLiberi = npostifila;
		pthread_mutex_init(&settori[i].file[j].mx_fila,NULL);
	}
}

/*
 * Inizializza tutti i settori dello stadio
 */
void inizializzaStadio(){
	int i;
	int nfile=0;
	int npostifila=0;
	int costoIntero=0;
	pthread_mutexattr_t attr;
	
	//inizializzo i settori
	for (i = 0; i < N_SETTORI; i++){
		switch(i){
			//settore i
			case CURVA_SUD: case CURVA_NORD:
				nfile = 35;
				npostifila = 30;
				costoIntero = 20;
				creaFilePosti(i, nfile, npostifila, costoIntero);
				break;
			case DISTINTI_SUD: case DISTINTI_OVEST:
			case DISTINTI_EST: case DISTINTI_NORD:
				nfile = 30;
				npostifila = 30;
				costoIntero = 28;
				creaFilePosti(i, nfile, npostifila, costoIntero);
				break;
			case TRIBUNA_TEVERE:
				nfile = 50;
				npostifila = 40;
				costoIntero = 55;
				creaFilePosti(i, nfile, npostifila, costoIntero);
				break;
			case TRIBUNA_MONTE_MARIO:
				nfile = 50;
				npostifila = 35;
				costoIntero = 90;
				creaFilePosti(i, nfile, npostifila, costoIntero);
				break;
			case TRIBUNA_ONORE:
				nfile = 20;
				npostifila = 20;
				costoIntero = 200;
				creaFilePosti(i, nfile, npostifila, costoIntero);
		}
		settori[i].codice = i;
		settori[i].postiDisponibili = nfile * npostifila;
		strcpy(settori[i].nome,nomiSettori[i]);
		pthread_mutexattr_init(&attr);
		pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK_NP);
		pthread_mutex_init(&settori[i].mx_aggiornamentoSettore,&attr);
		pthread_mutexattr_destroy(&attr);
	}
	printf("<SERVER-INFO> Stadio Inizializzato\n");
}

//stampa in una stringa le informazioni sul settore
void stampaDescrizioneSettore(char* stringa, struct settore* s){
	pthread_mutex_lock(&(s->mx_aggiornamentoSettore));
	sprintf(stringa,"Codice: %d, %s, posti liberi: %d, prezzo: %d, posti fila: %d\n",s->codice +1,s->nome,s->postiDisponibili, s->costoIntero, s->file[0].nPosti);
	pthread_mutex_unlock(&(s->mx_aggiornamentoSettore));
}

/* 
 * Algoritmo che prenota i posti nello stadio: appena trova un posto libero
 * in una fila del settore specificato dall'utente, verifica che ci siano n 
 * posti adiacenti accanto (dove n è specificato dal client); se ci sono
 * prenota tutto, altrimenti va avanti. Dopo aver scorso tutte le file,
 * se non trova n posti adiacenti restituisce 0 (esito negativo), altrimenti
 * 1. Ritorna -1 se nel settore non ci sono sufficienti posti.
 */
int prenotaPosti(int prenotazione[2], int client){
	int i,j,y;	//per ciclare su settore, fila e posto
	int adiacentiLiberi = 0;	//contatore    
	int prenotati = 0;	//risultato
	//estraggo i dati forniti dal client
	int nSettore = (prenotazione[0]);
	int postiAdiacentiRichiesti = prenotazione[1];
	
	//controllo se i posti sono sufficienti: non accadrà mai, perchè il controllo è
	//già effettuato in server.c
	pthread_mutex_lock(&settori[nSettore].mx_aggiornamentoSettore);
	if (settori[nSettore].postiDisponibili<postiAdiacentiRichiesti) {
		printf("<SERVER-INFO> Il client %i ha richiesto più posti di quelli disponibili nel settore %s\n",client, settori[nSettore].nome);
		return -1;
	}
	pthread_mutex_unlock(&settori[nSettore].mx_aggiornamentoSettore);
	
	//per tutte le file del settore
	for (i = 0; i < settori[nSettore].nFile; i++) {
		//acquisisettori[nSettore]sco il lock sulla fila
		pthread_mutex_lock(&settori[nSettore].file[i].mx_fila);
		//se la fila ha più posti di quelli richiesti e se ci sono posti a sufficienza
		if (settori[nSettore].file[i].nPostiLiberi >= postiAdiacentiRichiesti){ 
			//scandisco i posti della fila
			for (j = 0; j < settori[nSettore].file[i].nPosti; j++){
				//se il posto j non è stato ancora prenotato
				if (!settori[nSettore].file[i].posti[j].is_prenotato){
					for (y = j; y < (j+postiAdiacentiRichiesti); y++){
						//conto quanti posti liberi adiacenti ci sono
						if (!settori[nSettore].file[i].posti[y].is_prenotato){
							adiacentiLiberi++;
						}
					}
					//se bastano i posti liberi adiacenti
					if (adiacentiLiberi == postiAdiacentiRichiesti){
						for (y = j; y < (j+postiAdiacentiRichiesti); y++) {
							//prenoto il posto
							settori[nSettore].file[i].posti[y].is_prenotato == 1;
							//la fila ha un posto disponibile in meno
							settori[nSettore].file[i].nPostiLiberi--;
						}
						prenotati = 1;
						//aggiorno il settore
						pthread_mutex_lock(&settori[nSettore].mx_aggiornamentoSettore);
								settori[nSettore].postiDisponibili = settori[nSettore].postiDisponibili-postiAdiacentiRichiesti;
						pthread_mutex_unlock(&settori[nSettore].mx_aggiornamentoSettore);
						printf("<SERVER-UPDATE> Posti disponibili nel settore %d: %d\n",nSettore,settori[nSettore].postiDisponibili);
					}
					//se abbiamo trovato i posti, stop
					if (prenotati) 
						break;
					//altrimenti resetto
					adiacentiLiberi = 0;
				}
			}
		}
		pthread_mutex_unlock(&settori[nSettore].file[i].mx_fila);
			
		if (prenotati) 
			break;
	}		
	return prenotati;
}
